unit uFormMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, PngImage,
  VirtualUI_Settings;

type
  TFormMain = class(TForm)
    createProfileButton: TButton;
    hideAppTabButton: TButton;
    showAppTabButton: TButton;
    procedure createProfileButtonClick(Sender: TObject);
    procedure hideAppTabButtonClick(Sender: TObject);
    procedure showAppTabButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

procedure TFormMain.createProfileButtonClick(Sender: TObject);
var
  profiles: IProfiles;
  profile: IProfile;
  icon: TPngImage;
  virtualPath: string;
  p: integer;
begin
   virtualPath := 'demoapp';
   profile := nil;
   profiles := Server.Profiles;

    // Check if the profile to create already exists.
    // In this case, we've been using the VirtualPath which, if defined, must be unique.
    // We could check by a different field. for example, by the executable path and name.
    p := 0;
    while ((p < profiles.Count) and not Assigned(profile)) do
    begin
      if (profiles.Item[p].VirtualPath = virtualPath) then
          profile := profiles.Item[p]
      else
          Inc(p);
    end;
    if (not Assigned(profile)) then begin
      // Adds a new empty profile to the Server:
      profile := Server.Profiles.Add;
      // - VirtualPath must be unique.
      profile.VirtualPath   := 'demoapp';
    end;

  // Set basic profile properties:
  // - UserName and Password must be a valid Windows Account in the VirtualUI server machine:
  profile.Name          := 'Demo Application';
  profile.FileName      := 'C:\path_to_app\demo.exe';
  profile.UserName      := 'windows_user';
  profile.Password      := 'user_password';
  profile.ProfileKind   := PROFILE_App;
  //If needed, you can set other properties like Arguments, StartDir, etc.

  // Set profile icon, in this case from external file (you can also extract
  // the application icon, convert to PNG, and pass to IconToBase64):
  icon := TPngImage.Create;
  icon.LoadFromFile('demo_icon.png');
  profile.IconData := IconToBase64(icon);
  icon.Free;

  // Save changes:
  Server.Save;
end;

procedure TFormMain.hideAppTabButtonClick(Sender: TObject);
begin
  // This hides the "Applications" tab in the VirtualUI Server Manager:
  Server.HideSection(SRVSEC_Applications);

  // Save changes:
  Server.Save;
end;

procedure TFormMain.showAppTabButtonClick(Sender: TObject);
begin
  // This shows the "Applications" tab in the VirtualUI Server Manager:
  Server.ShowSection(SRVSEC_Applications);

  // Save changes:
  Server.Save;
end;

end.
